@@ -29,16 +29,25 @@
 #' @param fcontrol Character, either \code{"nospline"} (default, forcings are
 #'   handled by deSolve) or \code{"einspline"} (forcings are handled as splines
 #'   within the C code based on the einspline library).
-#' @param nGridpoints Integer, defining the number of grid points between tmin
-#'   and tmax where the ODE is computed in any case. Indicates also the number
-#'   of spline nodes if \code{fcontrol = "einspline"}.
+#' @param nGridpoints Integer, defining for which time points the ODE is evaluated
+#' or the solution is returned: Set \code{-1} to return only the explicitly requested
+#' time points (default). If additional time points are introduced through events, they
+#' will not be returned. Set \code{>= 0} to introduce additional time points between tmin
+#' and tmax where the ODE is evaluated in any case. Additional time points that might be
+#' introduced by events will be returned.
+#' If splines are used with \code{fcontrol = "einspline"}, \code{nGridpoinnts} also
+#' indicates the number of spline nodes.
+#' @param includeTimeZero Logical. Include t = 0 in the integration time points if \code{TRUE} 
+#' (default). Consequently,
+#' integration starts at t = 0 if only positive time points are provided by the user and at
+#' tmin, if also negtive time points are provided.
 #' @param precision Numeric. Only used when \code{fcontrol = "einspline"}.
 #' @param modelname Character. The C file is generated in the working directory
 #'   and is named <modelname>.c. If \code{NULL}, a random name starting with
 #'   ".f" is chosen, i.e. the file is hidden on a UNIX system.
 #' @param verbose Print compiler output to R command line.
 #' @param solver Select the solver suite as either \code{deSolve} or
-#'   \code{Sundials}. Defaults to \code{deSolve}.
+#'   \code{Sundials} (not available any more). Defaults to \code{deSolve}.
 #' @details The function replaces variables by arrays \code{y[i]}, etc. and
 #'   replaces "^" by pow() in order to have the correct C syntax. The file name
 #'   of the C-File is derived from \code{f}. I.e. \code{funC(abc, ...} will
@@ -48,6 +57,7 @@
 #'   
 #' @return the name of the generated shared object file together with a number
 #'   of attributes
+#'
 #' @examples 
 #' \dontrun{
 #' # Exponential decay plus constant supply
@@ -71,10 +81,14 @@ funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL,
                  jacobian=c("none", "full", "inz.lsodes", "jacvec.lsodes"), 
                  rootfunc = NULL, boundary = NULL, 
                  compile = TRUE, fcontrol = c("nospline", "einspline"),
-                 nGridpoints = 500, precision = 1e-5, modelname = NULL,
+                 nGridpoints = -1, includeTimeZero = TRUE, precision = 1e-5, modelname = NULL,
                  verbose = FALSE, solver = c("deSolve", "Sundials")) {
   
   f <- unclass(f)
+  
+  # If f contains line breaks, replace them by ""
+  f <- gsub("\n", "", f)
+  
   constraints <- NULL # Might be an interesting option in the future
   myattr <- attributes(f)
   equations <- f
@@ -134,6 +148,10 @@ funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL,
   
   deSolveSyntax <- function(f) {
     f <- replaceNumbers(f)
+    f <- gsub("++", "+", f, fixed = TRUE)
+    f <- gsub("+-", "-", f, fixed = TRUE)
+    f <- gsub("-+", "-", f, fixed = TRUE)
+    f <- gsub("--", "+", f, fixed = TRUE)
     f <- replaceOperation("^", "pow", f)
     f <- replaceOperation("**", "pow", f)
     f <- replaceSymbols(variables, paste0("y[", 1:length(variables) - 1, "]"), f)
@@ -166,10 +184,14 @@ funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL,
     eventsfn <- sapply(1:length(time.unique), function(i) {
       t <- time.unique[i]
       paste0("\t if(*t == ", t, " & eventcounter[", i-1, "] == 0) {\n",
-             paste(ifelse(method[time == t] == "replace", 
-                    paste0("\t\t", var[time == t], " = ", value[time == t], ";"),
-                    paste0("\t\t", var[time == t], " = ", var[time == t], " + ", value[time == t], ";")
-                    ), collapse = "\n"),
+             paste(sapply(which(time == t), function(tix) {
+               switch(
+                 method[tix],
+                 replace = paste0("\t\t", var[tix], " = ", value[tix], ";"),
+                 add = paste0("\t\t", var[tix], " = ", var[tix], " + ", value[tix], ";"),
+                 multiply = paste0("\t\t", var[tix], " = (", var[tix], ") * (", value[tix], ");")
+               )
+             }), collapse = "\n"),
              "\n",
              "\t\t", "eventcounter[", i-1, "] = 1.;\n",
              "\t }\n"
@@ -200,7 +222,7 @@ funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL,
   # Write to C file
   filename <- paste0(dllname, ".c")
   sink(filename)
-  cat("/** Code auto-generated by cOde", as.character(packageVersion("cOde")), "**/\n")
+  cat("/** Code auto-generated by cOde", as.character(utils::packageVersion("cOde")), "**/\n")
   cat(paste(includings, "\n"))
   cat("\n")
   cat(paste("static double parms[", dv+dp,"];\n", sep=""))
@@ -334,6 +356,8 @@ funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL,
     cat("/** Event function **/\n")
     cat(paste0("void ", modelname, "_myevent(int *n, double *t, double *y) {\n"))
     cat("\n")
+    cat("\t double time = *t;\n")
+    cat("\n")
     cat(eventsfn, sep = "\n")
     cat("\n")
     cat("}\n")
@@ -346,6 +370,7 @@ funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL,
     cat("/** Root function **/\n")
     cat(paste0("void ", modelname, "_myroot(int *neq, double *t, double *y, int *ng, double *gout, double *out, int *ip ) {\n"))
     cat("\n")
+    cat("\t double time = *t;\n")
     cat("\n")
     if(fcontrol == "einspline") {
       cat("\t double x[nSplines];\n")
@@ -407,40 +432,8 @@ funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL,
 
   
   } else if (solver == "Sundials") {
-    ## ---- Sundials::cvodes: write code ----
-    # Construct sensitivity equations
-    fSens <- sensitivitiesSymb(f, 
-                               states = setdiff(variables, fixed), 
-                               parameters = setdiff(parameters, fixed), 
-                               inputs = forcings,
-                               reduce = FALSE)
-    variablesSens <- names(fSens)
-
     
-    #### Assemble source code
-    dllname <- paste0(modelname, "_sdcv")
-    program <- c(sundialsIncludes(),
-                 sundialsOde(f, variables, parameters, dllname),
-                 sundialsSensOde(fSens, variables, variablesSens, parameters, dllname))
-    
-    if (jacobian != "none") {
-      program <- c(program,
-                   sundialsJac(f, variables, parameters, dllname))
-    }
-    
-    # First unload possible loaded DLL before writing C file (otherwise we have problems on Windows)
-    .so <- .Platform$dynlib.ext
-    test <- try(dyn.unload(paste0(dllname, .so)), silent = TRUE)
-    if (!inherits(test, "try-error")) message(paste("A shared object with name", dllname, "already existed and was overwritten."))
-    
-    
-    ## Write source code
-    filename <- paste0(dllname, ".cpp")
-    cppFile <- file(filename, open = "w")
-    writeLines(c(program,
-                 "}" # Matches extern "C" {
-    ), cppFile)
-    close(cppFile)
+    stop("Sundials support has been removed. If you were an active user of the Sundials implementation, please get in touch.")
     
   }
   
@@ -465,23 +458,13 @@ funC <- function(f, forcings = NULL, events = NULL, fixed = NULL, outputs=NULL,
   attr(f, "boundary") <- boundary
   attr(f, "rootfunc") <- rootfunc
   attr(f, "nGridpoints") <- nGridpoints
+  attr(f, "includeTimeZero") <- includeTimeZero
   attr(f, "fcontrol") <- fcontrol
   attr(f, "solver" ) <- solver
   attr(f, "modelname") <- modelname
   
-  if (solver == "Sundials") {
-    attr(f, "equationsSens") <- fSens
-    attr(f, "variablesSens") <- variablesSens
-    #attr(f, "adrDynamics") <- getNativeSymbolInfo("dynamics", PACKAGE = dllname)$address
-    #attr(f, "adrSensitivities") <- getNativeSymbolInfo("sensitivities", PACKAGE = dllname)$address
-    #attr(f, "adrDynamicsJac") <- if (jacobian != "none") getNativeSymbolInfo("dynamicsJac", PACKAGE = dllname)$address else NULL
-    # Attribute deriv is used in odeC to decide if sensitivities are to be
-    # integrated or not. As f is stored in func by odemodel(), deriv is set
-    # to false. odemodel() stores this f also in extended with deriv = TRUE.
-    attr(f, "deriv") <- FALSE 
-  }
-  
   return(f)
+  
 }
 
 
@@ -515,172 +498,6 @@ compileAndLoad <- function(filename, dllname, fcontrol, verbose) {
 
 
 
-#' Sundials::cvodes: Replace powers and symbols to get correct C++ syntax for
-#' cvodes.
-#' 
-#' @param f Named character vector.
-#' @param variables Variables appearing on the ODE, \code{names(f)}.
-#' @param parameters Parameters appearing in the ODE.
-#' @param varSym Symbol of the variables to appear in the source code.
-#' @param parSym Symbol of the parameter to appear in the source code.
-#' @param forcings Inhomogenity of the ODE. Currently not used.
-#'   
-#' @author Wolfgang Mader, \email{Wolfgang.Mader@@fdm.uni-freiburg.de}
-cvodesSyntax <- function(f, variables, parameters, varSym = "y", parSym = "p", forcings = NULL) {
-  f <- replaceOperation("^", "pow", f)
-  f <- replaceOperation("**", "pow", f)
-  f <- replaceSymbols(variables, paste0(varSym, "[", 1:length(variables) - 1, "]"), f)
-  f <- replaceSymbols(parameters, paste0(parSym, "[", 1:length(parameters) - 1, "]"), f)
-  return(f)
-}
-
-
-
-#' Infos and includes for sundials::cvodes C++ code: ODE, Jacobian.
-#' 
-#' @return C++ source code as a character vector.
-#'   
-#' @author Wolfgang Mader, \email{Wolfgang.Mader@@fdm.uni-freiburg.de}
-#' @importFrom utils packageVersion
-sundialsIncludes <- function() {
-  ## General preparations
-  #### User info
-  prolog <- paste("/**",
-                  paste0("** Code auto-generated by cOde ", as.character(packageVersion("cOde")), "."),
-                  "** Do not edit by hand.",
-                  "**/", sep = "\n")
-  
-  
-  #### Includes and function signature
-  includes <- paste("#include <array>",
-                    "#include <vector>",
-                    "#include <cmath>",
-                    "using std::array;",
-                    "using std::vector;", sep = "\n")
-  
-  
-  return(c(prolog, includes, "\n\n", 'extern "C" {\n'))
-}
-
-
-
-#' Implement the ODE system for sundials::cvodes.
-#' 
-#' @param f Named character vector containing the right-hand sides of the ODE.
-#'   You may use the key word.
-#' @param variables Variables appearing on the ODE, \code{names(f)}.
-#' @param parameters Parameters appearing in the ODE.
-#' @param modelname Base name of the dll.
-#' @return C++ source code as a character vector.
-#'   
-#' @author Wolfgang Mader, \email{Wolfgang.Mader@@fdm.uni-freiburg.de}
-sundialsOde <- function(f, variables, parameters, modelname) {
-  ## Header
-  odeHead <- paste("/** Derivatives **/",
-                    paste0("vector<double> ", modelname, "_dynamics(const double& time, const vector<double>& y,"),
-                    "                               const vector<double>& p, const vector<double>& f) {",
-                    "    vector<double> ydot(y.size());", sep = "\n")
-  
-  
-  ## ODE system
-  f <- cvodesSyntax(f, variables, parameters)
-  dv <- length(variables)
-
-  odeSystem <- paste0("    ydot[", 0:(dv - 1),"] = ", f,";")
-  
-  
-  ## Return statement
-  ret <- paste("    array<vector<double>, 2> output = {{ydot, ydot}};",
-                  "    return output;",
-                  "}", sep = "\n")
-  ret <- paste("    return ydot;",
-               "}", sep = "\n")
-  
-  
-  #### Return entire program
-  return(c(odeHead, odeSystem, ret, "\n"))
-}
-
-
-
-#' Implement sensitivities for an ODE system, sundials::cvodes.
-#' 
-#' @param f Named character vector containing the right-hand sides of the ODE. 
-#'   You may use the key word.
-#' @param variablesOde Variables appearing on the ODE of the dynamic system.
-#' @param variablesSens Variables appearing on the ODE of the sensitivities of
-#'   the dynamic system.
-#' @param parameters Parameters appearing in the ODE.
-#' @param modelname Base name of the dll.
-#' @return C++ source code as a character vector.
-#'   
-#' @author Wolfgang Mader, \email{Wolfgang.Mader@@fdm.uni-freiburg.de}
-sundialsSensOde <- function(f, variablesOde, variablesSens, parameters, modelname) {
-  ## Header
-  odeHead <- paste("/** Derivatives of sensitivities **/",
-                   paste0("vector<double> ", modelname, "_sensitivities (const double& time,"),
-                   "                              const vector<double>& y, const vector<double>& yS,",
-                   "                              const vector<double>& p) {",
-                   "    vector<double> ySdot(y.size() * (y.size() + p.size()));", sep = "\n")
-  
-  
-  ## Sensitivity ODE system
-  f <- cvodesSyntax(f, variablesOde, parameters)
-  f <- cvodesSyntax(f, variablesSens, parameters, "yS")
-  hasSens.idx <- c(attr(f, "hasSensStatesIdx"), attr(f, "hasSensParametersIdx"))
-  hasSens.position <- (1:length(hasSens.idx))[hasSens.idx]
-  sensOde <- paste0("    ySdot[", hasSens.position - 1,"] = ", f,";")
-  
-  
-  ## Return statement
-  ret <- paste("    return ySdot;",
-               "}", sep = "\n")
-  
-  
-  #### Return entire program
-  return(c(odeHead, sensOde, ret, "\n"))
-}
-
-
-
-#' Implement the Jacobian of the ODE system for sundials::cvodes.
-#' 
-#' @param f Named character vector containing the right-hand sides of the ODE.
-#'   You may use the key word.
-#' @param variables Variables appearing on the ODE, \code{names(f)}.
-#' @param parameters Parameters appearing in the ODE.
-#' @param modelname Base name of the dll.
-#' @return C++ source code as a character vector.
-#'   
-#' @author Wolfgang Mader, \email{Wolfgang.Mader@@fdm.uni-freiburg.de}
-sundialsJac <- function(f, variables, parameters, modelname) {
-  ## Header
-  jacHead <- paste("/** Jacobian **/",
-                   paste0("vector<double> ", modelname, "_dynamicsJac(const double& time, const std::vector<double>& y, "),
-                   "                        const std::vector<double>& p,",
-                   "                        const std::vector<double>& f) {",
-                   "    vector<double> yJac(y.size()*y.size());", sep = "\n")
-  
-  
-  ## Jacobian of the ODE system
-  # On the cvodes side, the jacobian is filled up column by column which is in
-  # line with the format of jac
-  jac  <- jacobianSymb(f)
-  jac <- cvodesSyntax(jac, variables, parameters)
-  not.zero.jac <- which(jac != "0")
-  jacobian <- paste0("    yJac[", not.zero.jac - 1,"] = ", jac[not.zero.jac],";")
-  
-  
-  ## Return statement
-  ret <- paste("    return yJac;",
-               "}", sep = "\n")
-  
-  
-  #### Return entire program
-  return(c(jacHead, jacobian, ret, "\n"))
-}
-
-
 
 #' Generate interpolation spline for the forcings and write into list of matrices
 #' 
@@ -705,8 +522,11 @@ setForcings <- function(func, forcings) {
   inputs <- attr(func, "forcings")
   fcontrol <- attr(func, "fcontrol")
   nGridpoints <- attr(func, "nGridpoints")
+  if (nGridpoints < 2 & fcontrol == "einspline")
+    stop("When using spline interpolation, nGridpoints (the number of spline nodes) must be >= 2.")
+  
   trange <- range(forcings$time)
-  tspan <- seq(trange[1], trange[2], len=nGridpoints)
+  tspan <- seq(trange[1], trange[2], len=max(2, nGridpoints))
   
   times <- NULL
   values <- NULL
@@ -751,7 +571,13 @@ setForcings <- function(func, forcings) {
 #' Interface to ode()
 #' 
 #' @param y named vector of type numeric. Initial values for the integration
-#' @param times vector of type numeric. Integration times
+#' @param times vector of type numeric. Integration times. If \code{includeTimeZero}
+#' is \code{TRUE} (see \link{funC}), the times vector is augmented by t = 0. If
+#' \code{nGridpoints} (see \link{funC}) was set >= 0, uniformly distributed time points
+#' between the first and last time point are introduced and the solution is returned
+#' for these time points, too. Any additional time points that are introduced during
+#' integration (e.g. event time points) are returned unless nGridpoints = -1 (the default).
+#' 
 #' @param func return value from funC()
 #' @param parms named vector of type numeric.
 #' @param ... further arguments going to \code{ode()}
@@ -759,143 +585,18 @@ setForcings <- function(func, forcings) {
 #' @return matrix with times and states
 #' @example inst/examples/example1.R
 #' @export
-#' @import Rcpp
-#' @useDynLib cOde
 odeC <- function(y, times, func, parms, ...) {
   
   # Sundials::cvodes
   if (attr(func, "solver") == "Sundials") {
     
+    stop("Sundials is not supported any more. If you were an active user of the Sundials implementation, please get in touch.")
     
-    if (any(is.na(y))) {
-      stop("At least one state is not provided with an initial value.")
-    }
-
-    if (any(is.na(parms))) {
-      stop("At least one parameter value is unspecified.")
-    }
-    
-
-    # Extract cvodes settings from ...
-    varargs <- list(...)
-    varnames <- names(varargs)
-    
-    
-    # Handle events
-    if (all(varnames != "events")) {
-      events <- data.frame()
-    }
-    else {
-      events <- varargs$events$data
-      # On the C++ side, we can't work with species names, only with their
-      # position in the systems of odes.
-      events$var <- match(events$var, names(y)) - 1
-      methods <- c("replace", "add", "multiply")
-      events$method <- match(events$method, methods)
-    }
-    
-    
-    settings <- list(rtol = 1e-6,
-                     atol = 1e-6,
-                     maxsteps = 1e3,
-                     maxord = 5,
-                     hini = 0,
-                     hmin = 0,
-                     hmax = 100,
-                     maxerr = 5,
-                     maxnonlin = 10,
-                     maxconvfail = 10,
-                     method = "bdf",
-                     jacobian = if (attr(func, "jacobian") == "none") FALSE
-                                else TRUE,
-                     minimum = -1e-4,
-                     positive = 1,
-                     which_states = length(attr(func, "equations")),
-                     which_observed = 0,
-                     stability = TRUE,
-                     sensitivities = if (attr(func, "deriv")) TRUE else FALSE
-                     )
-
-    userSettings <- intersect(names(settings), varnames)
-    settings[userSettings] <- varargs[userSettings]
-    
-    
-    
-    
-    # Check if a model is present
-    adrDynamics <- try(getNativeSymbolInfo(name = paste0(func, "_dynamics")), silent = TRUE)
-    if (inherits(adrDynamics, "try-error")) {
-      stop("No model provided. Create one by calling odemodel().")
-    }
-    
-    
-    # Check consistency of user setting and user provided jacobian.
-    if (settings["jacobian"] == TRUE ) {  
-      adrDynamicsJac <- try(getNativeSymbolInfo(name = paste0(func, "_dynamicsJac")), silent = TRUE)
-      if (inherits(adrDynamicsJac, "try-error")) {
-        stop("You said to provide the jacobian, but you did not.")
-      }
-    } else {
-      adrDynamicsJac <- NULL
-    }
-    
-    
-    # Check consistency of user setting and user provided sensitivities.
-    if (settings["sensitivities"] == TRUE ) {  
-      adrSensitivities <- try(getNativeSymbolInfo(name = paste0(func, "_sensitivities")), silent = TRUE)
-      if (inherits(adrSensitivities, "try-error")) {
-        stop("You said to provide sensitivities, but you did not.")
-      }
-    } else {
-      adrSensitivities <- NULL
-    }
-    
-    
-    # Setup sensitivities
-    # This is a pain, as deSolve and Sundials have different approaches.
-    # While the parameter y of odeC seems to provied initals for sensitivities,
-    # for Sundials they are not usefull, as only initials for sensitivities are
-    # reported which are not reduced by reduceSensitivities. Therefore, we have
-    # to construct from scratch.
-    # initSens must not be NULL when passed to wrap_cvodes
-    nStates <- length(attr(func, "equations"))
-    nPars <- length(parms)
-    initSens <- c(diag(nStates), matrix(0, nStates, nPars))
-    
-    
-    #cat("times:", times, "\n")
-    #cat("states:", y[names(attr(func, "equations"))], "\n")
-    #cat("parms:", parms[attr(func, "parameters")], "\n")
-
-    
-    # Call integrator
-    out = wrap_cvodes(times = times,
-                      states_ = y[names(attr(func, "equations"))],
-                      parameters_ = parms[attr(func, "parameters")],
-                      initSens_ = initSens,
-                      events_ = events,
-                      settings = settings,
-                      model_ = as.list(adrDynamics)$address,
-                      jacobian_ = as.list(adrDynamicsJac)$address,
-                      sens_ = as.list(adrSensitivities)$address)
-
-
-    # Prepare return
-    whichStates <- settings[["which_states"]]
-    whichObserved <- settings[["which_observed"]]
-    outnames <- c("time",
-                  attr(func, "variables")[0:whichStates],
-                  make.unique(rep("obs", whichObserved), sep = ""))
-    if (settings["sensitivities"] == TRUE) {
-      outnames <- c(outnames, attr(func, "variables")[-(1:whichStates + whichObserved)])
-    }
-    colnames(out) <- outnames
-
-    return(out)
   }
   
   ## deSolve ----------------------------------------------------------------------
   nGridpoints <- attr(func, "nGridpoints")
+  includeTimeZero <- attr(func, "includeTimeZero")
   modelname <- attr(func, "modelname")
   
   # Evaluate initial values and parameters
@@ -918,9 +619,18 @@ odeC <- function(y, times, func, parms, ...) {
   
   
   #times <- sort(union(times, eventlist[["time"]]))
-  times.inner <- seq(min(c(times, 0)), max(times), len=nGridpoints)
-  times.inner <- sort(unique(c(times, times.inner)))
-  which.times <- match(times, times.inner)
+  times.outer <- times.inner <- times
+  if (includeTimeZero)
+    times.inner <- union(times.inner, 0)
+  if (nGridpoints > 0)
+    times.inner <- union(
+      times.inner, 
+      seq(min(times.inner), max(times.inner), len=nGridpoints)
+    )
+  
+  times.inner <- sort(unique(times.inner))
+  which.times <- match(times.outer, times.inner)
+  
   yout <- c(attr(func, "forcings"), names(attr(func, "outputs")))
   
   arglist <- list(y = y, times = times.inner, func = paste0(func, "_derivs"), parms = parms, dllname = modelname, initfunc = paste0(func, "_initmod"))
@@ -972,9 +682,15 @@ odeC <- function(y, times, func, parms, ...) {
   #loadDLL(func)
   
   out <- do.call(deSolve::ode, arglist)
-  out <- out[out[, 1] >= min(times.inner) & out[, 1] <= max(times.inner), ]
-  #out.index <- unique(c(which.times[which.times <= nrow(out)], nrow(out)))
-  #out <- matrix(out[out.index, ], nrow = length(out.index), dimnames = list(NULL, colnames(out)))
+  
+  if (nGridpoints == -1) {
+    # Return only requested time points
+    out <- out[match(times.outer, out[, 1]), , drop = FALSE]
+  } else {
+    # Return all time points between tmin and tmax (additional time points after
+    # tmax might be introduced by the integrator)
+    out <- out[out[, 1] >= min(times.inner) & out[, 1] <= max(times.inner), ]  
+  }
   
   return(out)
   
@@ -1035,8 +751,6 @@ bvptwpC <- function(yini=NULL, x, func, yend=NULL, parms, xguess=NULL, yguess=NU
   if(any(names(moreargs)=="forcings") & attr(func, "fcontrol") == "einspline") 
     moreargs <- moreargs[-which(names(moreargs)=="forcings")]
 
-  print(initforc)
-   
   out <- do.call(bvpSolve::bvptwp, c(moreargs, list(
     x = x, parms = newparms, xguess = xguess, yguess = yguess, posbound=posbound,
     func = paste0(func, "_derivs"), jacfunc = paste0(func, "_jacobian"), bound = paste0(func, "_gsub"), jacbound = paste0(func, "_dgsub"), 
